function [W,hloc] = SABRweights2Drbffd(r,rho,alpha,beta,xscale,s,N,n,m,p,indin,phi,ep,adap,parallel)
% Constructs a SABR differentiation matrix W from 2D grid s,
%stencil size n and indices indin.

% if nargin == 12
%     adap = 'reg';
%     parallel = 0;
% elseif nargin == 11
%     parallel = 0;
% end

% if parallel
%     ppar = gcp();
%     argfor = ppar.NumWorkers;
% else
%     ppar = gcp('nocreate');
% %     delete(p);
%     argfor = 0;
% end

    ppar = gcp('nocreate');
    argfor = ppar.NumWorkers;


indc = findKNearestNeighbors(s,s,n);

iind = repmat(indin,n,1); iind = iind(:); %n*N
jind = transpose(indc(indin,:)); jind = jind(:);%n*N
Wval = zeros(n,numel(indin));  %n*N

% parfor (ii = indin, argfor)
hloc = zeros(size(s,1),1);
parfor (jj = 1:numel(indin), argfor)
    warning off
    ii = indin(jj);
    sc = s(ii,:); xc = sc(:,1); yc = sc(:,2);
    se = s(indc(ii,:),:);
    
    Rc = xcdist(se,se,1);
%     H = Rc(:,:,1);
%     hloc(jj) = min(H(H>0));
    
    A = RBFmat(phi,ep,Rc,'0',1);
    
    %     Ax = RBFmat(phi,ep,Rc,'1',1);
    %     Ay = RBFmat(phi,ep,Rc,'1',2);
    
    Axx = RBFmat(phi,ep,Rc,'2',1);
    Ayy = RBFmat(phi,ep,Rc,'2',2);
    Axy = RBFmat(phi,ep,Rc,'m2',1:2);
    
    [P,lP] = vander2D(xscale,sc,se,n,p,r,rho,alpha,beta); % defined below!
    
    
    Ac = [A, P;
        P', zeros(m,m)];
    
    lA = transpose(-r*A(1,:)...
        + 0.5 * (yc'.^2) .* xc'.^(2*beta) * xscale^(2*(beta-1)) .* Axx(1,:)...
        + 0.5 * alpha^2 * (yc'.^2) .* Ayy(1,:)...
        + rho * alpha * xc'^beta .* (yc'^2) * xscale^(beta-1) .* Axy(1,:));
    
    lc = [lA; lP];
    
    wc = Ac\lc;
    
    Wval(:,jj) = wc(1:n);
end

Wval = Wval(:);
W = sparse(iind,jind,Wval,N,N);
end

function [P, lP] = vander2D(xscale,sc,s,n,p,r,rho,alpha,beta)
x = s(:,1);
y = s(:,2);

xc = sc(1);
yc = sc(2);

dim = 2;
m = nchoosek(p+dim, p); %number of polynomial terms;

P = zeros(n,m);
lP = zeros(m,1);

kk = 0;
for ii = 0:p
    for jj = 0:ii
        kk = kk+1;
        
        i = ii-jj;
        j = jj;
        
        P(:,kk) = x.^(i) .* y.^(j);
        
        lP(kk) = xc^(i)*yc^(j) * (-r + 0.5*xscale^(2*(beta-1))*i*(i-1)*xc^(2*beta-2)*yc^2 + rho*alpha*xscale^(beta-1)*i*j*xc^(beta-1)*yc + 0.5*alpha^2*j*(j-1));
        %         lP(kk) = xc^(i)*yc^(j) * (-r +r*(i+j) + 0.5*(i-1)*i*sig1^2 + 0.5*(j-1)*j*sig2^2 +i*j*rho*sig1*sig2);
        
    end
end
end